(* ****************************************************************************
 *                     FKUtl, Fumio Kawamata's Utility
 *                  Copyright (C) 1998-2008 Fumio Kawamata
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
 *  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.
 * ****************************************************************************
 *)
unit FKUtl;

interface

uses
  Forms;

type
  (* Version information
   *)
  TVersionInfo = Class(TObject)
    private
      FCompanyName: String;
      FFileDescription: String;
      FFileVersion: String;
      FInternalName: String;
      FLegalCopyright: String;
      FLegalTradeMarks: String;
      FOriginalFileName: String;
      FProductName: String;
      FProductVersion: String;
      FComments: String;
    public
      property CompanyName: String read FCompanyName;
      property FileDescription: String read FFileDescription;
      property FileVersion: String read FFileVersion;
      property InternalName: String read FInternalName;
      property LegalCopyright: String read FLegalCopyright;
      property LegalTradeMarks: String read FLegalTradeMarks;
      property OriginalFileName: String read FOriginalFileName;
      property ProductName: String read FProductName;
      property ProductVersion: String read FProductVersion;
      property Comments: String read FComments;
      constructor Create;
      destructor Destroy; override;
      procedure RetrieveItems;
      procedure DisplayItems;
    end;

procedure ReadFormStatusFromIniFile(Form: TForm; IniFileFullPath: String);
procedure WriteFormStatusToIniFile(Form: TForm; IniFileFullPath: String);

procedure ReadFontInfoFromIniFile(Form: TForm; IniFileFullPath: String);
procedure WriteFontInfoToIniFile(Form: TForm; IniFileFullPath: String);

function SplitBeforeDelimiter(var S: string; Delimiter: Char): string;

implementation

uses
  Windows, Dialogs, IniFiles, Messages, Controls, Graphics, SysUtils ;

var
  Msg: TMessage = (Msg: CM_PARENTFONTCHANGED);
  (* Msg is for the FontInformation procedures *)
  (* TMessage requires Messages unit. *)
  (* CM_PARENTFONTCHANGED requires Controls unit. *)


constructor TVersionInfo.Create;
begin
  FCompanyName:='';
  FFileDescription:='';
  FFileVersion:='';
  FInternalName:='';
  FLegalCopyright:='';
  FOriginalFileName:='';
  FProductName:='';
  FProductVersion:='';
  FComments:='';
end;


destructor TVersionInfo.Destroy;
begin
  inherited;
end;


procedure TVersionInfo.RetrieveItems;
const
  Language     = '0411'; (* Japanese *)
  CharacterSet = '03A4'; (* Windows, Japanse(Shift JIS, X-0208) *)
var
  FullInfoSize: DWORD;
  ItemInfoSize: DWORD;
  pFullInfo: Pointer;
  pItemInfo: Pointer;
  Ret: DWORD;
begin
  FullInfoSize:=GetFileVersionInfoSize(pChar(Application.ExeName),Ret);
  if FullInfoSize>0 then begin
    GetMem(pFullInfo,FullInfoSize);
    try
      GetFileVersionInfo(PChar(Application.ExeName),0,FullInfoSize,pFullInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\CompanyName'),pItemInfo,ItemInfoSize);
      FCompanyName:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\FileDescription'),pItemInfo,ItemInfoSize);
      FFileDescription:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\InternalName'),pItemInfo,ItemInfoSize);
      FInternalName:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\FileVersion'),pItemInfo,ItemInfoSize);
      FFileVersion:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\LegalCopyright'),pItemInfo,ItemInfoSize);
      FLegalCopyright:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\LegalTradeMarks'),pItemInfo,ItemInfoSize);
      FLegalTradeMarks:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\OriginalFileName'),pItemInfo,ItemInfoSize);
      FOriginalFileName:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\ProductName'),pItemInfo,ItemInfoSize);
      FProductName:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\ProductVersion'),pItemInfo,ItemInfoSize);
      FProductVersion:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\Comments'),pItemInfo,ItemInfoSize);
      FComments:=PChar(pItemInfo);
    finally
      FreeMem(pFullInfo,FullInfoSize);
    end;
  end
end;

procedure TVersionInfo.DisplayItems;
const
  CRLF = chr($0D)+chr($0A);
var
  Mess : String;
begin
  Mess:='Product name: '+FProductName+CRLF
       +'Original file name: '+FOriginalFileName+CRLF
       +'File description: '+FFileDescription+CRLF
       +'File version: '+FFileVersion+CRLF
       +'Legal Copyright: '+FLegalCopyright+CRLF
       +'Comment: '+FComments+CRLF;
  MessageDlg(Mess,mtInformation,[mbOK],0);
end;


procedure ReadFormStatusFromIniFile(Form: TForm; IniFileFullPath: String);
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  Form.Top:=AppIni.ReadInteger(Form.Name,'Top', Form.Top);
  Form.Left:=AppIni.ReadInteger(Form.Name,'Left', Form.Left);
  Form.Width:=AppIni.ReadInteger(Form.Name,'Width', Form.Width);
  Form.Height:=AppIni.ReadInteger(Form.Name,'Height', Form.Height);
  Form.WindowState:=TWindowState(AppIni.ReadInteger(Form.Name,'WindowState', 0));
  AppIni.Free;
end;

procedure WriteFormStatusToIniFile(Form: TForm; IniFileFullPath: String);
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  AppIni.WriteInteger(Form.Name,'WindowState', Integer(Form.Windowstate));
  AppIni.WriteInteger(Form.Name,'Top', Form.Top);
  AppIni.WriteInteger(Form.Name,'Left', Form.Left);
  AppIni.WriteInteger(Form.Name,'Width', Form.Width);
  AppIni.WriteInteger(Form.Name,'Height', Form.Height);
  AppIni.UpdateFile;
  AppIni.Free;
end;



procedure ReadFontInfoFromIniFile(Form: TForm; IniFileFullPath: String);
var
  AppIni : TMemIniFile;
  I:         Integer;
  FontStyle: Byte;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  Form.Font.Name:=AppIni.ReadString('Font', 'FontName', 'Courier New');
  FontStyle:=AppIni.ReadInteger('Font', 'FontStyle', 0);
  Form.Font.Style:=TFontStyles(FontStyle);
  Form.Font.Size:=AppIni.ReadInteger('Font', 'FontSize', 9);
  Form.Font.Charset:=TFontCharSet(AppIni.ReadInteger('Font', 'CharacterSet', 0));
  for I:= 0 to Screen.FormCount-1 do
  begin
    //Screen.Forms[I].Dispatch(Msg);
    Screen.Forms[I].Font:=Form.Font;
  end;
  AppIni.Free;
end;


procedure WriteFontInfoToIniFile(Form: TForm; IniFileFullPath: String);
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  AppIni.WriteString('Font', 'FontName', Form.Font.Name);
  AppIni.WriteInteger('Font', 'FontStyle', Byte(Form.Font.Style));
  AppIni.WriteInteger('Font', 'FontSize', Byte(Form.Font.Size));
  AppIni.WriteInteger('Font', 'CharacterSet', Byte(Form.Font.Charset));
  AppIni.UpdateFile;
  AppIni.Free;
end;



(* *****************************************************************************
 * SplitBeforeDelimiter
 * This function requirs SysUtils unit for trim function.
 * *****************************************************************************
 *)
function SplitBeforeDelimiter(var S: string; Delimiter: Char): string;
var
  DelimiterPos: Integer;
begin
  DelimiterPos:=Pos(Delimiter, S);
  if DelimiterPos=0 then
  begin
    Result:=Trim(S);
    S:='';
  end
  else
  if DelimiterPos=1 then
  begin
    Result:='';
    Delete(S, 1, 1);
  end
  else
  begin
    Result:=Trim(Copy(S, 1, DelimiterPos-1));
    Delete(S, 1, DelimiterPos);
  end;
end;

end.
